@@ -1,30 +1,28 @@
 --
 -- polyglossia-tibt.lua
--- part of polyglossia v1.66 -- 2023/12/11
+-- part of polyglossia v2.6 -- 2025/06/30
 --
 
 require('polyglossia') -- just in case...
 
 local add_to_callback = luatexbase.add_to_callback
 local remove_from_callback = luatexbase.remove_from_callback
+local declare_callback_rule = luatexbase.declare_callback_rule
 local priority_in_callback = luatexbase.priority_in_callback
 
 local next, type = next, type
-
-local nodes, fonts, node = nodes, fonts, node
-
-local nodecodes          = nodes.nodecodes --- <= preloaded node.types()
+local node = node
 
 local insert_node_before = node.insert_before
 local insert_node_after  = node.insert_after
-local remove_node        = nodes.remove
+local remove_node        = node.remove
 local copy_node          = node.copy
 local has_attribute      = node.has_attribute
 
 local end_of_math        = node.end_of_math
 if not end_of_math then -- luatex < .76
   local traverse_nodes = node.traverse_id
-  local math_code      = nodecodes.math
+  local math_code      = node.id('math_char')
   local end_of_math = function (n)
     for n in traverse_nodes(math_code, n.next) do
       return n
@@ -32,10 +30,10 @@ if not end_of_math then -- luatex < .76
   end
 end
 
--- node types as of April 2013
-local glyph_code         = nodecodes.glyph
-local penalty_code       = nodecodes.penalty
-local kern_code          = nodecodes.kern
+-- node types as of April 2024
+local glyph_code         = node.id('glyph')
+local penalty_code       = node.id('penalty')
+local kern_code          = node.id('kern')
 
 -- we make a new node, so that we can copy it later on
 local penalty_node  = node.new(penalty_code)
@@ -66,7 +64,6 @@ local function process(head)
                 end
             end
         elseif id == math_code then
-            -- warning: this is a feature of luatex > 0.76
             start = end_of_math(start) -- weird, can return nil .. no math end?
         end
         if start then
@@ -79,15 +76,13 @@ end
 local callback_name = "pre_linebreak_filter"
 
 local function activate()
-  if not priority_in_callback (callback_name, "polyglossia-tibt.process") then
-    add_to_callback(callback_name, process, "polyglossia-tibt.process", 1)
-  end
+    add_to_callback(callback_name, process, "polyglossia-tibt.process")
+    declare_callback_rule(callback_name,
+    "polyglossia-tibt.process", "before", "luaotfload.node_processor")
 end
 
 local function desactivate()
-  if priority_in_callback (callback_name, "polyglossia-tibt.process") then
-    remove_from_callback(callback_name, "polyglossia-tibt.process")
-  end
+	remove_from_callback(callback_name, "polyglossia-tibt.process")
 end
 
 polyglossia.activate_tibt_eol    = activate
