@@ -1,6 +1,6 @@
 -- luatexko.lua
 --
--- Copyright (c) 2013-2021 Dohyun Kim <nomos at ktug org>
+-- Copyright (c) 2013-2025 Dohyun Kim <nomosnomos at gmail com>
 --                         Soojin Nam <jsunam at gmail com>
 --
 -- This work may be distributed and/or modified under the
@@ -13,8 +13,8 @@
 
 luatexbase.provides_module {
   name        = 'luatexko',
-  date        = '2021/07/10',
-  version     = '3.3',
+  date        = '2025/07/16',
+  version     = '4.1',
   description = 'typesetting Korean with LuaTeX',
   author      = 'Dohyun Kim, Soojin Nam',
   license     = 'LPPL v1.3+',
@@ -304,7 +304,7 @@ local fontoptions = {
       if has_harf_data(fid) then
         newwd = getparameters(fid) or false
         if newwd then
-          newwd = { newwd.space, newwd.space_stretch, newwd.space_shrink }
+          newwd = { newwd.space, newwd.space_stretch, newwd.space_shrink, newwd.extra_space }
         end
       else
         local newsp = nodenew(glyphid)
@@ -312,7 +312,7 @@ local fontoptions = {
         newsp = nodes.simple_font_handler(newsp)
         newwd = newsp and newsp.width or false
         if newwd then
-          newwd = { texsp(newwd), texsp(newwd/2), texsp(newwd/3) }
+          newwd = { texsp(newwd), texsp(newwd/2), texsp(newwd/3), texsp(newwd/3) }
         end
         if newsp then nodefree(newsp) end
       end
@@ -548,10 +548,10 @@ local special_classes = {
     [0xFF01] = 4, [0xFF1A] = 2, [0xFF1B] = 2, [0xFF1F] = 4,
   }, { __index = charclass }),
   setmetatable({  -- TC
-    [0x3001] = 3, [0x3002] = 5, [0xFF0C] = 3, [0xFF0E] = 5,
+    [0x3001] = 3, [0x3002] = 3, [0xFF0C] = 3, [0xFF0E] = 3,
   }, { __index = charclass }),
   setmetatable({  -- TC vert
-    [0x3001] = 3, [0x3002] = 5, [0xFF0C] = 3, [0xFF0E] = 5,
+    [0x3001] = 3, [0x3002] = 3, [0xFF0C] = 3, [0xFF0E] = 3,
     [0xFF1A] = 7, [0xFF1B] = 7,  -- 0xFE13, 0xFE14
   }, { __index = charclass }),
   setmetatable({  -- JP vert
@@ -668,40 +668,13 @@ function luatexko.updateforcehangul (value)
   nodewrite(what)
 end
 
-local function hangul_space_skip (curr, newfont)
-  if curr.lang ~= nohyphen and curr.font ~= newfont then
-    -- fontloader's "node" mode sets space_stretch to zero
-    -- when the font is a monospaced font (fontspec's \setmonofont
-    -- command does the same thing), which we will bypass here
-    -- for alignment of CJK and Latin glyphs in verbatim environment.
-    -- See http://www.ktug.org/xe/index.php?document_srl=249772
-    if not fontoptions.monospaced[curr.font] then
-      local n = getnext(curr)
-      if n and n.id == glueid and n.subtype == spaceskip then
-        local params = getparameters(curr.font)
-        local oldwd, oldst, oldsh, oldsto, oldsho = getglue(n)
-        if params
-          and oldwd == params.space
-          and oldst == params.space_stretch
-          and oldsh == params.space_shrink
-          and oldsto == 0
-          and oldsho == 0 then -- not user's spaceskip
-
-          local newwd = fontoptions.hangulspaceskip[newfont]
-          if newwd then
-            setglue(n, newwd[1], newwd[2], newwd[3])
-          end
-        end
-      end
-    end
-  end
-end
-
 local function process_fonts (head)
-  local curr = head
+  local curr, currfont, currlang, newfont = head, 0, nohyphen, 0
+  local to_free = { }
   while curr do
     local id = curr.id
     if id == glyphid then
+      currfont, currlang = curr.font, curr.lang
       if curr.font ~= 0 -- exclude nullfont
         and not has_attribute(curr, unicodeattr) then
 
@@ -712,7 +685,6 @@ local function process_fonts (head)
           local p = getprev(curr)
           if p and p.id == glyphid then
             if curr.font ~= p.font then
-              hangul_space_skip(curr, p.font)
               curr.font = p.font
             end
             curr.lang = p.lang
@@ -747,16 +719,13 @@ local function process_fonts (head)
             then
             curr.font = hf
           elseif hf and has_attribute(curr, hangulbyhangulattr) and is_hangul_jamo(c) then
-            hangul_space_skip(curr, hf)
             curr.font = hf
           elseif hjf and has_attribute(curr, hanjabyhanjaattr) and is_hanja(c) then
-            hangul_space_skip(curr, hjf)
             curr.font = hjf
           elseif not char_in_font(curr.font, c) then
             local fbf = has_attribute(curr, fallbackfontattr) or false
             for _,f in ipairs{ hf, hjf, fbf } do
               if f and char_in_font(f, c) then
-                hangul_space_skip(curr, f)
                 curr.font = f
                 break
               end
@@ -765,6 +734,46 @@ local function process_fonts (head)
           set_attribute(curr, unicodeattr, c)
         end
       end
+      newfont = curr.font
+    elseif id == glueid
+      and currfont ~= 0
+      and currfont ~= newfont
+      and currlang ~= nohyphen
+      and curr.subtype == spaceskip
+      -- fontloader's "node" mode sets space_stretch to zero
+      -- when the font is a monospaced font (fontspec's \setmonofont
+      -- command does the same thing), which we will bypass here
+      -- for alignment of CJK and Latin glyphs in verbatim environment.
+      -- See http://www.ktug.org/xe/index.php?document_srl=249772
+      and not fontoptions.monospaced[currfont] then
+
+      local params = getparameters(currfont)
+      local oldwd, oldst, oldsh, oldsto, oldsho = getglue(curr)
+      if params and oldsto == 0 and oldsho == 0 then
+        local p = getprev(curr)
+        local sf = p and p.char and tex.getsfcode(p.char) or 1000
+        if sf == 0 or sf > 1000 then
+          local p, pf = getprev(p), 0
+          while p and pf == 0 do
+            pf = p.char and tex.getsfcode(p.char) or 1000
+            p = getprev(p)
+          end
+          if sf == 0 then sf = pf end
+          if pf < 1000 then sf = 1000 end
+        end
+        if oldwd == (sf < 2000 and params.space or params.space+params.extra_space)
+          and oldst == texsp(params.space_stretch * (sf/1000))
+          and oldsh == texsp(params.space_shrink * (1000/sf)) then
+
+          local newwd = fontoptions.hangulspaceskip[newfont]
+          if newwd then
+            setglue(curr,
+                    sf < 2000 and newwd[1] or newwd[1]+newwd[4],
+                    texsp(newwd[2] * (sf/1000)),
+                    texsp(newwd[3] * (1000/sf)))
+          end
+        end
+      end
     elseif id == discid then
       process_fonts(curr.pre)
       process_fonts(curr.post)
@@ -779,9 +788,14 @@ local function process_fonts (head)
       if type(value) == "function" then
         value()
       end
+
+      noderemove(head, curr)
+      tableinsert(to_free, curr)
     end
     curr = getnext(curr)
   end
+
+  for _, v in ipairs(to_free) do nodefree(v) end
 end
 
 -- linebreak
@@ -862,6 +876,9 @@ local function insert_glue_before (head, curr, par, br, brb, classic, ict, dim,
     setglue(gl, dim, str, str*0.6)
   end
 
+  local prev = getprev(curr)
+  gl.attr = prev and prev.attr or curr.attr -- for less tagging
+
   head = insert_before(head, curr, pn)
   return insert_before(head, curr, gl)
 end
@@ -1071,7 +1088,8 @@ local function process_glyph_width (head)
           gpos = gpos and gpos.id == kernid and gpos.subtype == fontkern
 
           if not gpos then
-            local wd = fontoptions.en_size[curr.font] - curr.width
+            local diff = char_in_font(curr.font, curr.char).luatexko_diff or 0
+            local wd = fontoptions.en_size[curr.font] - (curr.width + diff)
             if wd ~= 0 then
               local k = nodenew(kernid) -- fontkern (subtype 0) is default
               k.kern = class == 3 and wd/2 or wd
@@ -1348,6 +1366,7 @@ end
 
 local function process_dotemph (head)
   local curr = head
+  local to_free = { }
   while curr do
     if curr.list then
       curr.list = process_dotemph(curr.list)
@@ -1355,25 +1374,26 @@ local function process_dotemph (head)
     elseif curr.id == glyphid then
       local dotattr = has_attribute(curr, dotemphattr)
       if dotattr and dotemphbox[dotattr] then
+        unset_attribute(curr, dotemphattr) -- avoid multiple run
 
-        local ok
+        local init
         if hangul_tonemark[curr.char] and harf_reordered_tonemark(curr) then
           curr = getnext(curr)
           if is_hangul_jamo(has_attribute(curr, unicodeattr) or curr.char) then
-            ok = true
+            init = curr
           end
         else
           if not is_combining(curr.char) then -- bypass unicodeattr inherited
             local c = has_attribute(curr, unicodeattr) or curr.char
             if is_hangul(c) or is_compat_jamo(c) or is_chosong(c) or is_hanja(c) or is_kana(c) then
-              ok = true
+              init = curr
             end
           end
         end
 
-        if ok then
-          local currwd = curr.width
-          if currwd >= fontoptions.en_size[curr.font] then
+        if init then
+          local basewd = curr.width
+          if basewd >= fontoptions.en_size[curr.font] then
             local box = nodecopy(dotemphbox[dotattr]).list
             -- bypass unwanted nodes injected by some other packages
             while box.id ~= hlistid do
@@ -1381,7 +1401,30 @@ local function process_dotemph (head)
               box = getnext(box)
             end
 
-            local shift = (currwd - box.width)/2
+            -- put the dot after base syllable
+            local n = getnext(curr)
+            while n do
+              if n.id == glyphid then
+                local c = has_attribute(n, unicodeattr) or n.char
+                if is_jungsong(c) or is_jongsong(c) or is_combining(c) then
+                  basewd = basewd + n.width
+                else
+                  break
+                end
+              elseif n.id == kernid and n.subtype == fontkern then
+                basewd = basewd + n.kern
+              elseif n.id == whatsitid
+                and  n.mode == directmode
+                and  my_node_props(n).endactualtext then
+                -- pass
+              else
+                break
+              end
+              curr = n
+              n = getnext(n)
+            end
+
+            local shift = (basewd - box.width)/2 - basewd
             if shift ~= 0 then
               local list = box.list
               local k = nodenew(kernid)
@@ -1390,10 +1433,10 @@ local function process_dotemph (head)
             end
 
             -- consider charraise
-            box.shift = shift_put_top(curr, box)
+            box.shift = shift_put_top(init, box)
 
             box.width = 0
-            head = insert_before(head, curr, box)
+            head, curr = insert_after(head, curr, box)
           end
         end
       end
@@ -1405,9 +1448,14 @@ local function process_dotemph (head)
       local val = curr.value
       nodefree(dotemphbox[val])
       dotemphbox[val] = nil
+
+      tableinsert(to_free, curr)
+      head = noderemove(head, curr)
     end
     curr = getnext(curr)
   end
+
+  for _, v in ipairs(to_free) do nodefree(v) end
   return head
 end
 
@@ -1485,6 +1533,7 @@ local function draw_uline (head, curr, parent, t, final)
       setglue(g, len)
       g.subtype = subtype
       g.leader  = final and list or nodecopy(list)
+      g.attr = curr.attr
       local k = nodenew(kernid)
       k.kern = -len
       head = insert_before(head, start, g)
@@ -1498,6 +1547,7 @@ local ulitems = {}
 
 local function process_uline (head, parent, level)
   local curr, level = head, level or 0
+  local to_free = { }
   while curr do
     if curr.list then
       curr.list = process_uline(curr.list, curr, level+1)
@@ -1521,14 +1571,11 @@ local function process_uline (head, parent, level)
         end
       end
 
-      local to_free = curr
-      head, curr = noderemove(head, curr)
-      nodefree(to_free)
-      goto nextnode
+      tableinsert(to_free, curr)
+      head = noderemove(head, curr)
 
     end
     curr = getnext(curr)
-    ::nextnode::
   end
 
   for _, item in pairs(ulitems) do
@@ -1538,6 +1585,7 @@ local function process_uline (head, parent, level)
     end
   end
 
+  for _, v in ipairs(to_free) do nodefree(v) end
   return head
 end
 
@@ -1700,6 +1748,7 @@ local function process_reorder_tonemarks (head)
 
             local actual    = pdfliteral_direct_actual(syllable)
             local endactual = pdfliteral_direct_actual()
+            actual.attr, endactual.attr = TM.attr, TM.attr -- for tagged pdf
             head = insert_before(head, init, actual)
             head, curr = insert_after(head, curr, endactual)
 
@@ -1714,6 +1763,7 @@ local function process_reorder_tonemarks (head)
           if fontoptions.tonemark_xmax[curr.font] >= 0 then
             local actual    = pdfliteral_direct_actual{ init = curr, uni }
             local endactual = pdfliteral_direct_actual()
+            actual.attr, endactual.attr = dotcircle.attr, dotcircle.attr -- for tagged pdf
             head = insert_before(head, curr, actual)
             head, curr = insert_after(head, curr, dotcircle)
             head, curr = insert_after(head, curr, endactual)
@@ -1905,13 +1955,14 @@ local function process_vertical_font (fontdata)
   local goffset = xheight/2 * (dfltfntsize / quad) -- TODO?
 
   -- declare shift amount of horizontal box inside vertical env.
+  if fontdata.vertcharraise then return end -- avoid multiple running
   fontdata.vertcharraise = goffset
 
   for i,v in pairs(fontdata.characters) do
     local voff = goffset - (v.width or 0)/2
     local bbox = descriptions[i] and descriptions[i].boundingbox or {0,0,0,0}
     local gid  = v.index
-    local tsb  = tsb_tab[gid].tsb
+    local tsb  = tsb_tab[gid] and tsb_tab[gid].tsb
     local hoff = tsb and (bbox[4] + tsb) * scale or ascender
     v.commands = {
       { "down", -voff },
@@ -1922,8 +1973,15 @@ local function process_vertical_font (fontdata)
       { "pop" },
       { "pdf", "Q" },
     }
-    local vw = tsb_tab[gid].ht
-    v.width  = vw and vw * scale or quad
+    local vw = tsb_tab[gid] and tsb_tab[gid].ht
+    vw = vw and vw * scale or quad
+
+    -- character width shall be consistent with the width in the font program
+    local diff = vw - v.width
+    if diff and diff ~= 0 then
+      v.luatexko_diff = diff
+    end
+
     local ht = bbox[3] * scale + voff
     local dp = bbox[1] * scale + voff
     v.height = ht > 0 and  ht or nil
@@ -1931,9 +1989,11 @@ local function process_vertical_font (fontdata)
   end
   local spacechar = char_in_font(fontdata, 32)
   if spacechar then
-    parameters.space         = spacechar.width
-    parameters.space_stretch = spacechar.width/2
-    parameters.space_shrink  = spacechar.width/2
+    local wd = spacechar.width or parameters.space
+    wd = wd + (spacechar.luatexko_diff or 0)
+    parameters.space         = wd
+    parameters.space_stretch = wd/2
+    parameters.space_shrink  = wd/2
   end
   parameters.ascender  = quad/2 + goffset
   parameters.descender = quad/2 - goffset
@@ -1973,6 +2033,26 @@ local function process_vertical_font (fontdata)
   end
 end
 
+local function process_vertical_diff (head)
+  local curr = head
+  while curr do
+    if curr.id == glyphid then
+      local attr = has_attribute(curr, classicattr)
+      if attr == 1 or attr == 4 or attr == 5 then
+        local chardata = char_in_font(curr.font, curr.char)
+        local diff = chardata and chardata.luatexko_diff
+        if diff and diff ~= 0 then
+          local k = nodenew(kernid)
+          k.kern = diff
+          head, curr = insert_after(head, curr, k)
+        end
+      end
+    end
+    curr = getnext(curr)
+  end
+  return head
+end
+
 function luatexko.gethorizboxmoveright ()
   for _, v in ipairs{ fontcurrent(),
                       texattribute.luatexkohangulfontattr,
@@ -2165,9 +2245,9 @@ end, "luatexko.pre_shaping_filter")
 
 local otfregister = fonts.constructors.features.otf.register
 
-local function activate_process (cbnam, cbfun, name)
+local function activate_process (cbnam, cbfun, name, first)
   if not active_processes[name] then
-    add_to_callback(cbnam, cbfun, "luatexko."..cbnam.."."..name)
+    add_to_callback(cbnam, cbfun, "luatexko."..cbnam.."."..name, first)
     active_processes[name] = true
   end
 end
@@ -2263,7 +2343,10 @@ otfregister {
   description = "vertical typesetting",
   default = false,
   manipulators = {
-    node = process_vertical_font,
+    node = function(fontdata)
+      process_vertical_font(fontdata)
+      activate_process("post_shaping_filter", process_vertical_diff, "verticalwriting", 1)
+    end,
     plug = function(fontdata)
       local fullname = fontdata.fullname
       fontdata_warning("vertical."..fullname,
@@ -2284,10 +2367,12 @@ otfregister {
       end
     end,
     plug = function(fontdata, _, value)
+      ---[[ adapt to luaotfload
       local setup = fonts.expansions.setups[value] or {}
       fontdata.stretch = fontdata.stretch or (setup.stretch or 2)*10
       fontdata.shrink  = fontdata.shrink  or (setup.shrink  or 2)*10
       fontdata.step    = fontdata.step    or (setup.step    or .5)*10
+      --]]
       if tex.adjustspacing == 0 then
         texset("global", "adjustspacing", 2)
       end
@@ -2315,6 +2400,11 @@ local function get_HB_variant_char (fontdata, charcode)
   end
 end
 
+fonts.protrusions.setups.default[0xFF0C] = { 0, 1 }
+fonts.protrusions.setups.default[0xFF0E] = { 0, 1 }
+fonts.protrusions.setups.default[0xFF1A] = { 0, 1 }
+fonts.protrusions.setups.default[0xFF1B] = { 0, 1 }
+
 otfregister {
   name = "protrusion",
   description = "glyph protrusion",
@@ -2323,14 +2413,16 @@ otfregister {
     node = function(fontdata, _, value)
       local setup = fonts.protrusions.setups[value] or {}
       local quad  = fontdata.parameters.quad
+      local left,right,factor = setup.left or 1, setup.right or 1, setup.factor or 1
       for i, v in pairs(fontdata.characters) do
         local uni = v.unicode
         if uni then
           local lr = setup[uni]
           if lr then
-            local wdq = v.width/quad*1000
-            v.left_protruding  = wdq*lr[1]
-            v.right_protruding = wdq*lr[2]
+            local wdq = (v.width + (v.luatexko_diff or 0))/quad*1000
+            local l, r = lr[1], lr[2]
+            if l and l ~= 0 then v.left_protruding  = wdq*l*left*factor end
+            if r and r ~= 0 then v.right_protruding = wdq*r*right*factor end
           end
         end
       end
@@ -2341,13 +2433,18 @@ otfregister {
     plug = function(fontdata, _, value)
       local setup = fonts.protrusions.setups[value] or {}
       local quad  = fontdata.parameters.quad
+      local chrs  = fontdata.characters
+      local left,right,factor = setup.left or 1, setup.right or 1, setup.factor or 1
       for i, v in pairs(setup) do
-        for _, ii in ipairs{i, get_HB_variant_char(fontdata,i)} do
-          local chr = fontdata.characters[ii]
-          if chr then
-            local wdq = chr.width/quad*1000
-            chr.left_protruding  = wdq*v[1]
-            chr.right_protruding = wdq*v[2]
+        if chrs[i] then
+          local l, r = v[1], v[2]
+          for _, ii in ipairs{i, get_HB_variant_char(fontdata,i)} do
+            local chr = chrs[ii]
+            if chr then
+              local wdq = chr.width/quad*1000
+              if l and l ~= 0 then chr.left_protruding  = wdq*l*left*factor end
+              if r and r ~= 0 then chr.right_protruding = wdq*r*right*factor end
+            end
           end
         end
       end
